/*
 LSM303DLH: 3-axis Accelerometer / 3-axis Magnetometer Library for Arduino
   created by NeoCat <neocat@neocat.jp>
*/

#include "LSM303DLH.h"
LSM303DLH_Class LSM303DLH;

// internal function - send a register value
void LSM303DLH_Class::_write_reg(byte id, byte addr, byte val)
{
/*
  Serial.print(id, HEX);
  Serial.print(" ");
  Serial.print(addr, HEX);
  Serial.print("=");
  Serial.println(val, HEX);
*/
  Wire.beginTransmission(id);
  Wire.send(addr);
  Wire.send(val);
  Wire.endTransmission();
}

// internal function - receive a register value
byte LSM303DLH_Class::_read_reg(byte id, byte addr)
{
  Wire.beginTransmission(id);
  Wire.send(addr);
  Wire.endTransmission();
  
  Wire.requestFrom((int)id, 1);
  while (!Wire.available());
  return Wire.receive();
}


// internal function - receive 6 values from specified id/addr
void LSM303DLH_Class::get_value(byte id, byte addr, bool little_endian, short val[3])
{
  Wire.beginTransmission(id);
  Wire.send(addr); // send start address
  Wire.endTransmission();

  Wire.requestFrom((int)id, 6);
  for (int i = 0; i < 3; i++) {
    while (!Wire.available());
    val[i] = ((short)Wire.receive()) << (little_endian ? 0 : 8);
    while (!Wire.available());
    val[i] |= ((short)Wire.receive()) << (little_endian ? 8 : 0);
  }
}

// Put sensor into sleep mode
void LSM303DLH_Class::stop()
{
  write_reg(0x20, 0x07); // CTRL_REG1_A: power-down
  write_reg(0x02, 0x03); // MR_REG_M: sleep mmode
}

// Get Accelerometer value (x/y/z: -32768 - 32767)
void LSM303DLH_Class::get_accel(short val[3])
{
  get_value(0x18 | sa,  // accel (SA=0/1)
            0x28|0x80,  // OUT_X_L | Multiple bytes access bit
            true,       // L->H (little endian)
            val);
}

// Get Magnetometer value (x/y/z: -4096 - 4095)
void LSM303DLH_Class::get_magnetic(short val[3])
{
  get_value(0x1e,  // magnetic sensor
            0x03,  // OUT_X_M (pointer is automatically updated)
            false, // H->L (big endian)
            val);
}

