import processing.serial.*;
import processing.opengl.*;

Serial port;
PFont font;
int x, y, z;
int mx, my, mz;

void setup() {
  size(400,400,OPENGL);
  frameRate(60);
  smooth();
  rectMode(CENTER);

  font = loadFont("Monaco-24.vlw");
  textFont(font, 16);
  textAlign(RIGHT);

  //String portName = Serial.list()[0];
  port = new Serial(this,"/dev/tty.usbserial-A7006UFY",9600);
  port.clear();
  port.bufferUntil(10);  

}

void serialEvent(Serial p) {
  String stringData = port.readStringUntil(10);

  if (stringData != null) {
    stringData = trim(stringData);
    int data[] = int(split(stringData,' '));
    if (data.length >= 6) {
      x=data[0];
      y=data[1];
      z=data[2];
      mx=data[3];
      my=data[4];
      mz=data[5];
    }
  }  
}


void draw() {
  background(6);
  stroke(255);
  lights();

  // display sensor values
  fill(63,127,255);
  text(x,100,20);
  text(y,200,20);
  text(z,300,20);

  fill(255,127,63);
  text(mx,100,40);
  text(my,200,40);
  text(mz,300,40);

  // calculation of rotation matrix
  float a = sqrt(x*x+y*y+z*z);
  float m = sqrt(mx*mx+my*my+mz*mz);

  float hx = my*z - mz*y;
  float hy = mz*x - mx*z;
  float hz = mx*y - my*x;
  float h = sqrt(hx*hx + hy*hy + hz*hz);

  float rx = y*hz - z*hy;
  float ry = z*hx - x*hz;
  float rz = x*hy - y*hx;
  float r = sqrt(rx*rx + ry*ry + rz*rz);

  // display heading info in degrees
  float head = atan2(hx/h, rx/r);
  text(int(degrees(head)), 100,70);

  // display data using vector
  translate(200,80);
  fill(63,127,255);
  beginShape();
  vertex(0,0,0);
  vertex(0,10,0);
  vertex(x/a*20, y/a*20, z/a*20);
  endShape(CLOSE);

  fill(255,127,63);
  beginShape();
  vertex(0,0,0);
  vertex(0,10,0);
  vertex(mx/m*20, my/m*20, mz/m*20);
  endShape(CLOSE);
  translate(-200,-80);

  // setup coordinates for board picture
  translate(width/2,height/2);
  rotateX(radians(90)); // move Z-axis up
  rotateZ(radians(90)); // move X-axis front
  
  // rotate by the matrix
  applyMatrix(hx/h, hy/h, hz/h, 0.0,
              -rx/r, -ry/r, -rz/r, 0.0,
              x/a,  y/a,  z/a,  0.0,
              0.0,  0.0,  0.0,  1.0);

  // Draw board
  scale(2.0);
  fill(255,63,63);
  box(50, 50, 10);
  translate(0,0,5);
  fill(0,0,0);
  box(10, 10, 3);
  translate(0,0,-5);
  translate(28,0,0);
  fill(255,0,0);
  box(4,10,10);
}

